@@ -1,11 +1,11 @@
 ;;; q-mode.el --- A q editing mode    -*- lexical-binding: t -*-
 
-;; Copyright (C) 2006-2025 Nick Psaris <nick.psaris@gmail.com>
+;; Copyright (C) 2006-2026 Nick Psaris <nick.psaris@gmail.com>
 ;; Keywords: faces files q
 ;; Package-Requires: ((emacs "24"))
 ;; Created: 8 Jun 2015
-;; Package-Version: 20251115.2320
-;; Package-Revision: b42778533328
+;; Package-Version: 20260101.2301
+;; Package-Revision: 72dea8631501
 ;; URL: https://github.com/psaris/q-mode
 
 ;; This file is not part of GNU Emacs.
@@ -28,16 +28,15 @@
 ;; A major mode for editing q (the language written by Kx Systems, see
 ;; URL `https://code.kx.com') in Emacs.
 
-;; A Flymake backend using the "q" program is also provided.
-
-;;
 ;; Some of its major features include:
 ;;
-;;  - syntax highlighting (font lock),
+;;  - syntax highlighting (font-lock-mode),
 ;;
-;;  - interaction with inferior q[con] instance,
+;;  - syntax checking (flymake-mode),
 ;;
-;;  - scans declarations and places them in a menu.
+;;  - interaction with inferior q[con] instance (comint-mode),
+;;
+;;  - variable and function indexing (imenu).
 ;;
 ;; To load `q-mode' on-demand, instead of at startup, add this to your
 ;; initialization file
@@ -87,6 +86,10 @@
 ;; `C-c M-l' can be used to load the file associated with the active
 ;; buffer.
 
+;; Quick access to variable and function definitions can be obtained
+;; using the `imenu' binding `M-g i'.
+
+
 ;; `M-x customize-group' can be used to customize the `q' group.
 ;; Specifically, the `q-program' and `q-qcon-program' variables can be
 ;; changed depending on your environment.
@@ -294,6 +297,8 @@ command to read the command line arguments from the minibuffer."
          (buffer (get-buffer-create (format "*%s*" (q-shell-name host port))))
          (command (if qs "ssh" (getenv "SHELL")))
          (switches (append (if qs (list "-t" host) (list "-c")) (list cmd)))
+         ;; disable kdb-x rlwrap functionality
+         (process-environment (cons "KX_LINE=0" process-environment))
          process)
     (when (called-interactively-p 'any) (pop-to-buffer buffer))
     (when (or current-prefix-arg (not (comint-check-proc buffer)))
@@ -412,12 +417,43 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
   (interactive)
   (q-eval-region (point-min) (point-max)))
 
+
+(defvar q-symbol-regex
+  "`\\(?:\\(?:\\w\\|[.]\\)\\(?:\\w\\|[_.]\\)*\\)?"
+  "Regular expression used to find symbols.")
+
+(defvar q-file-regex
+  (concat q-symbol-regex ":\\(?:\\w\\|[/:_.]\\)*")
+  "Regular expression used to find files.")
+
+(defvar q-name-regex
+  "\\_<\\([.]?[a-zA-Z]\\(?:\\w\\|[_.]\\)*\\)\\s-*"
+  "Regular expression used to find variable or function names.")
+
 (defvar q-function-regex
-  "\\_<\\([.]?[a-zA-Z]\\(?:\\s_\\|\\w\\|_\\)*\\s-*\\)::?\\s-*\\(?:{\\|'\\s-*\\[\\|[^;{\n]*?\\(?:::\\|[-.~=!@#$%^&*_+|,<>?/\\:']\\)\\s-*\\(?:\\s<\\|$\\|;\\)\\)"
+  (concat q-name-regex
+          ":"                           ; assignment
+          ":?"                          ; view
+          "\\s-*"                       ; potential white space
+          "\\(?:"                       ; one of the following
+          "{"                           ; function declaration
+          "\\|'\\s-*\\["                ; composition
+          "\\|[^;{\n]*?\\(?:::\\|[-.~=!@#$%^&*_+|,<>?/\\:']" ; trailing binary operator
+          "\\)"
+          "\\s-*"                       ; potential white space
+          "\\(?:\\s<\\|$\\|;\\)"        ; opening comment, new line, or semicolon
+          "\\)"
+          )
   "Regular expression used to find function declarations.")
 
 (defvar q-variable-regex
-    "\\_<\\([.]?[a-zA-Z]\\(?:\\s_\\|\\w\\|_\\)*\\s-*\\)[-.~=!@#$%^&*_+|,<>?]?:"
+  (concat q-name-regex
+          "[-.~=!@#$%^&*_+|,<>?]?"      ; potential compound assignment
+          ":"                           ; assignment
+          ":?"                          ; view
+          "\\s-*"                       ; potential space
+          "[^ )}:;\n]"                  ; something else
+          )
   "Regular expression used to find variable declarations.")
 
 (defun q-eval-function ()
@@ -523,24 +559,22 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
 
 (defvar q-keywords
   (eval-when-compile
-    (regexp-opt
-     '("abs" "acos" "asin" "atan" "avg" "bin" "binr" "by" "cor" "cos" "cov" "dev" "delete"
-       "div" "do" "enlist" "exec" "exit" "exp" "from" "getenv" "hopen" "if" "in" "insert" "last"
-       "like" "log" "max" "min" "prd" "select" "setenv" "sin" "sqrt" "ss"
-       "sum" "tan" "update" "var" "wavg" "while" "within" "wsum" "xexp") `words))
-  "Keywords for q mode.")
-
-(defvar q-type-words
-  (eval-when-compile
-    (concat "\\(`"
-            (regexp-opt '("boolean" "byte" "short" "long" "real" "int" "float" "char" "symbol"
-                          "month" "date" "datetime" "minute" "second" "time" "timespan" "timestamp"
-                          "year" "mm" "dd" "hh" "uu" "ss" "week") t) "\\)\\s-*[$]"))
-  "Types for q mode.")
+    (concat "\\_<"
+            "\\(?:[_]\\)?"              ; leading _ is not a symbol
+            (regexp-opt
+             '("abs" "acos" "asin" "atan" "avg" "bin" "binr" "by" "cor" "cos" "cov" "dev" "delete"
+               "div" "do" "enlist" "exec" "exit" "exp" "from" "getenv" "hopen" "if" "in" "insert" "last"
+               "like" "log" "max" "min" "prd" "select" "setenv" "sin" "sqrt" "ss"
+               "sum" "tan" "update" "var" "wavg" "while" "within" "wsum" "xexp") t)
+            "\\_>"))
+  "Keywords for q mode defined in .Q.res.")
 
 (defvar q-builtin-words
   (eval-when-compile
-    (concat "\\_<\\(?:[.]q[.]\\)?"
+    (concat "\\_<"
+            "\\(?:[_]\\)?"              ; leading _ is not a symbol
+            "\\("
+            "\\(?:[.]q[.]\\)?"
             (regexp-opt
              '( "aj" "aj0" "ajf" "ajf0" "all" "and" "any" "asc" "asof" "attr" "avgs" "ceiling"
                 "cols" "count" "cross" "csv" "cut" "deltas" "desc"
@@ -553,50 +587,59 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
                 "prev" "rand" "rank" "ratios" "raze" "read0" "read1" "reciprocal" "reval"
                 "reverse" "rload" "rotate" "rsave" "rtrim" "save" "scan" "scov" "sdev" "set" "show"
                 "signum" "ssr" "string" "sublist" "sums" "sv" "svar" "system" "tables" "til"
-                "trim" "type" "uj" "ujf" "ungroup" "union" "upper" "upsert" "value"
+                "trim" "type" "uj" "ujf" "ungroup" "union" "upper" "upsert" "use" "value"
                 "view" "views" "vs" "where" "wj" "wj1" "ww" "xasc" "xbar" "xcol" "xcols" "xdesc"
-                "xgroup" "xkey" "xlog" "xprev" "xrank")) "\\_>"))
-  "Builtin functions defined in q.k.")
+                "xgroup" "xkey" "xlog" "xprev" "xrank"))
+            "\\)"
+            "\\_>"))
+  "Builtin functions for q mode defined in q.k.")
 
-(defvar q-constant-words
+(defvar q-builtin-dot-z-words
   (eval-when-compile
-    (regexp-opt '(".z.D" ".z.K" ".z.T" ".z.Z" ".z.N" ".z.P" ".z.a" ".z.b" ".z.d" ".z.exit" ".z.f"
-                  ".z.h" ".z.i" ".z.k" ".z.l" ".z.o" ".z.pc" ".z.pg" ".z.ph" ".z.pi"
-                  ".z.po" ".z.pp" ".z.ps" ".z.pw" ".z.s" ".z.t" ".z.ts" ".z.u" ".z.vs"
-                  ".z.w" ".z.x" ".z.z" ".z.n" ".z.p" ".z.ws" ".z.bm") `symbols))
-  "Constants for q mode.")
+    (concat "\\_<"
+            "\\(?:[_]\\)?"              ; leading _ is not a symbol
+            (regexp-opt '(".z.D" ".z.H" ".z.K" ".z.N" ".z.P" ".z.T" ".z.W" ".z.X" ".z.Z" ".z.a"
+                          ".z.ac" ".z.b" ".z.bm" ".z.c" ".z.d" ".z.e" ".z.ex" ".z.exit"
+                          ".z.ey" ".z.f" ".z.h" ".z.i" ".z.k" ".z.l" ".z.n" ".z.o" ".z.p"
+                          ".z.pc" ".z.pd" ".z.pg" ".z.ph" ".z.pi" ".z.pm" ".z.po" ".z.pp"
+                          ".z.pq" ".z.ps" ".z.pw" ".z.q" ".z.r" ".z.s" ".z.t" ".z.ts" ".z.u"
+                          ".z.vs" ".z.w" ".z.wc" ".z.wo" ".z.ws" ".z.x" ".z.z" ".z.zd") t)
+            "\\_>"))
+  "Builtin .z functions/constants defined for q mode.")
 
-(defvar q-font-lock-keywords
-  (list '("^\\\\\\_<.*?$" 0 font-lock-constant-face keep) ; lines starting with a '\' are compile time
-        (list q-function-regex 1 font-lock-function-name-face nil) ; functions
-        )
+(defvar q-font-lock-keywords ;; keywords
+  (list
+   ;; q single-letter system commands keep comments
+   '("^\\\\\\(?:\\w\\|[_]\\)\\(?:\\s-.*?\\)?$" 0 font-lock-preprocessor-face keep)
+   ;; os multi-letter system commands ignore comments
+   '("^\\\\\\w\\w.*?$" 0 font-lock-preprocessor-face prepend)
+   '("^'.*" . font-lock-warning-face) ; error
+   (list (concat "[; ]\\('" q-symbol-regex "\\)") 1 font-lock-warning-face nil) ; signal
+   (cons q-file-regex 'font-lock-preprocessor-face) ; files
+   (cons q-symbol-regex 'font-lock-constant-face) ; symbols
+   )
   "Minimal highlighting expressions for q mode.")
 
-(defvar q-font-lock-keywords-1 ; types
+(defvar q-font-lock-keywords-1          ; symbols
   (append q-font-lock-keywords
           (list
-           '("`:\\(?:\\w\\|[/:._]\\)*" . font-lock-preprocessor-face) ; files
-           '("\\(`\\_<[gpsu]\\)#" 1 font-lock-type-face nil) ; attributes
-           '("^'.*?$" 0 font-lock-warning-face nil)   ; error
-           '("[; ]\\('`\\w*\\)" 1 font-lock-warning-face nil) ; signal
-           '("`\\(?:\\(?:\\w\\|[.]\\)\\(?:\\s_\\|\\w\\|_\\)*\\)?" . font-lock-constant-face) ; symbols
-           '("\\b[0-2]:" . font-lock-preprocessor-face) ; IO/IPC
-           (list q-type-words 1 font-lock-type-face nil) ; `minute`year
-           (cons q-keywords  'font-lock-keyword-face)   ; select from
-           (cons q-builtin-words 'font-lock-builtin-face) ; q.k
+           (list q-keywords 1 'font-lock-keyword-face nil) ; select from
+           '("\\b[0-2]:" . font-lock-builtin-face)         ; IO/IPC
+           (list q-builtin-words 1 'font-lock-builtin-face nil) ; q.k
+           (list q-builtin-dot-z-words 1 'font-lock-builtin-face nil) ; .z.*
            ))
   "More highlighting expressions for q mode.")
 
-(defvar q-font-lock-keywords-2 ; keywords & literals
+(defvar q-font-lock-keywords-2 ; function/variable names and literals
   (append q-font-lock-keywords-1
           (list
-           (cons q-constant-words 'font-lock-constant-face) ; .z.*
+           (list q-function-regex 1 'font-lock-function-name-face nil) ; functions
            (list q-variable-regex 1 font-lock-variable-name-face nil) ; variables
            '("\\_<[0-9]\\{4\\}\\.[0-9]\\{2\\}\\(?:m\\|\\.[0-9]\\{2\\}\\(?:T\\(?:[0-9]\\{2\\}\\(?::[0-9]\\{2\\}\\(?::[0-9]\\{2\\}\\(?:\\.[0-9]*\\)?\\)?\\)?\\)?\\)?\\)\\_>" . font-lock-constant-face) ; month/date/datetime
            '("\\_<\\(?:[0-9]\\{4\\}\\.[0-9]\\{2\\}\\.[0-9]\\{2\\}\\|[0-9]+\\)D\\(?:[0-9]\\(?:[0-9]\\(?::[0-9]\\{2\\}\\(?::[0-9]\\{2\\}\\(?:\\.[0-9]*\\)?\\)?\\)?\\)?\\)?\\_>" . font-lock-constant-face) ; timespan/timestamp
            '("\\_<[0-9a-f]\\{8\\}-[0-9a-f]\\{4\\}-[0-9a-f]\\{4\\}-[0-9a-f]\\{4\\}-[0-9a-f]\\{12\\}\\_>" . font-lock-constant-face) ; guid
            '("\\_<[0-9]\\{2\\}:[0-9]\\{2\\}\\(?::[0-9]\\{2\\}\\(?:\\.[0-9]\\|\\.[0-9]\\{2\\}\\|\\.[0-9]\\{3\\}\\)?\\)?\\_>" . font-lock-constant-face) ; time
-           '("\\<[0-9]*[0-9.][0-9]*\\(?:[eE][+-]?[0-9]+\\)?[ef]?\\>" . font-lock-constant-face) ; floats/reals
+           '("\\_<\\(?:[0-9]+\\.\\(?:[0-9]*\\)?\\|\\.[0-9]+\\)\\(?:[eE][+-]?[0-9]+\\)?[ef]?\\_>" . font-lock-constant-face) ; floats/reals
            '("\\_<[0-9]+[cefhijnptuv]?\\_>" . font-lock-constant-face) ; char/real/float/short/int/long/time-types
            '("\\_<[01]+b\\_>" . font-lock-constant-face) ; bool
            '("\\_<0x[0-9a-fA-F]+\\_>" . font-lock-constant-face) ; bytes
@@ -624,8 +667,8 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
     (modify-syntax-entry ?\/ ".  " table) ; treat / as punctuation
     (modify-syntax-entry ?\n ">  " table) ; comments are ended by a new line
     (modify-syntax-entry ?\r ">  " table) ; comments are ended by a new line
-    (modify-syntax-entry ?\. "_  " table) ; treat . as a symbol
-    (modify-syntax-entry ?\_ ".  " table) ; treat _ as punctuation
+    (modify-syntax-entry ?\. "_  " table) ; treat _ as symbol
+    (modify-syntax-entry ?\_ "_  " table) ; treat _ as symbol
     (modify-syntax-entry ?\\ ".  " table) ; treat \ as punctuation
     (modify-syntax-entry ?\$ ".  " table) ; treat $ as punctuation
     (modify-syntax-entry ?\% ".  " table) ; treat % as punctuation
@@ -708,13 +751,11 @@ updates when a buffer is saved."
   (setq comint-prompt-regexp "^\\(q)+\\|[^:]*:[0-9]+>\\)")
   (setq font-lock-defaults q-font-lock-defaults)
   (set (make-local-variable 'comint-process-echoes) nil)
-  ;;  (set (make-local-variable 'comint-process-echoes) (not (equal q-host "")))
-  (set (make-local-variable 'comint-password-prompt-regexp) "[Pp]assword")
   )
 
 (defvar q-imenu-generic-expression
   (list
-   (list nil (concat "^" q-variable-regex) 1))
+   (list nil (concat "^" q-name-regex ":") 1))
   "Regular expressions to get q expressions into imenu.")
 
 ;;;###autoload
